# SPDX-FileCopyrightText: © Antonio López Rivera <antonlopezr99@gmail.com>
# SPDX-License-Identifier: GPL-3.0-only

"""
CLI
---
"""


import argparse

from emblem import coverage


def add(cli, name, help='', nargs=1):
    cli.add_argument(
        f'--{name}',
        nargs=nargs,
        default=None,
        help=help
    )


def main():

    CLI = argparse.ArgumentParser()

    CLI.add_argument(
        'coverage',
        help='code coverage percentage'
    )

    # Emblem arguments
    e_args = {'label':    ['badge label'],
              'style':    ['badge style'],
              'logo':     ['badge logo'],
              'fname':    ['svg file name'],
              'colors':   ['colors from which to generate a linear segmented colormap, low to high', '*'],
              'cmap':     ['matplotlib colormap']}
    # Runtime arguments
    r_args = {'silence':  ['print shields.io url']}

    _args = {**e_args, **r_args}
    for arg in _args.keys():
        add(CLI, arg, *_args[arg])

    # Parse input
    _input = CLI.parse_args()
    
    # Create input dictionary
    kwargs = {}

    for arg in e_args.keys():
        if getattr(_input, arg) is not None:
            v = getattr(_input, arg)
            if isinstance(v, list) and len(v) == 1:
                v = v[0]
            kwargs[arg] = v

    # Emblem
    q = coverage(_input.coverage, **kwargs)

    if not _input.silence:
        print(f'\n{q}\n')


if __name__ == '__main__':
    main()


# SPDX-FileCopyrightText: © Antonio López Rivera <antonlopezr99@gmail.com>
# SPDX-License-Identifier: GPL-3.0-only

"""
Utilities
---------
"""


import requests


def save_svg(url, fname):

    svg = requests.get(url).text

    with open(fname, 'w') as f:
        f.write(svg)



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="emblem",
    version="1.0.3",
    author="Antonio Lopez Rivera",
    author_email="antonlopezr99@gmail.com",
    description="shields.io + colormaps",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/alopezrivera/emblem",
    entry_points={
        "console_scripts": [
            "emblem = emblem.__main__:main",
        ],
    },
    packages=setuptools.find_packages(),
    install_requires=[
        "matplotlib>=3.3.4",
        "requests>=2.27.1"
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)



# SPDX-FileCopyrightText: © Antonio López Rivera <antonlopezr99@gmail.com>
# SPDX-License-Identifier: GPL-3.0-only

"""
Coverage
--------
"""


import re

from matplotlib.cm import get_cmap
from matplotlib.colors import to_rgb, to_hex, LinearSegmentedColormap

from emblem.utilities import save_svg


def custom_map(
    colors= ['darkred', 'lightgreen'],
    weights=None
    ):

    if weights is not None: colors = list(zip(weights, colors))

    m = LinearSegmentedColormap.from_list(colors=colors, 
                                          name='coverageMap')

    return m


def coverage(
    coverage,
    label= 'Coverage',
    style= 'for-the-badge',
    logo=  'pytest',
    fname ='./coverage.svg',
    colors=['#b00909',
            '#ff8f05',
            '#adadad',
            '#3faefc',
            '#3ade65'],
    cmap=  None
    ):
    """
    ``label``
    Badge label

    ``style``
    Badge style

    ``logo``
    Badge logo to be chosen from the standard set below 
    or simple-icons (https://simpleicons.org/).
        
        * bitcoin
        * dependabot
        * gitlab
        * npm
        * paypal
        * serverfault
        * stackexchange
        * superuser
        * telegram
        * travis    
    """

    # Determine contents of coverage string
    percentage = re.findall('\d+\.*\d*', coverage)

    if not percentage: 
        percentage = 0
    else:
        percentage = float(percentage[0])
        coverage = f'{percentage} %25'

    if cmap is not None:
        cmap = get_cmap(cmap)
    else:
        cmap = custom_map(colors)

    color = to_hex(cmap(percentage/100))[1:]
    
    # Syntax
    style = f"?style={style}" if style else ""
    logo  = f"&logo={logo}" if logo else ""
    
    shield = f'https://img.shields.io/badge/{label}-{coverage}-{color}{style}{logo}'.replace(' ', '%20')

    save_svg(shield, fname)

    return shield
